/*---------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 * ---------------------------------------------------------------------------
 */

#include <stdio.h>
#include <reg5000.h>
#include <absacc.h>
#include <stdlib.h>
#include <string.h>
#include "ds8007.h"
#include "LCD_Funct.h"

#define CPU_CRYSTAL 11059000//18432000*2
#define BAUD_RATE 57600//115200
#define BAUDDOUBLE 1
#define InitTrans 0
#define CreditTrans 1
#define DebitTrans 2

/* define bit location for SW4 */
sbit	P32 = 0xB2;				// P32/INT = Port 3.2

// Called-function prototypes
void serialSetup(void);
void dispTrans(void);
void doTrans(int slot);
void delay(int secs);
void transError(void);
uint8_t initCard(void);
uint8_t selDelay(int secs);
uint16_t selectFile(void);
uint16_t readRecords(void);
uint16_t transATR(void);
uint16_t transDebit(void);
uint16_t transCredit(void);
uint16_t transEnd(void);

uint8_t LCD_Str[42];

void main(void)
{
	int slot = 0;
  	uint8_t	val, temp;

	// Initialize the serial port, and output a startup message
	EA=0;
	serialSetup();
	puts("\n\nACOS3 Transaction Demo Example\n");
	val = dssc_readregister(CSR);
	printf("DS8007 Version: %bx\n\n",val>>4);

	// Reset DS8007 UART and deselect all cards
	dssc_init();

	// Initialize LCD Module, and display 2-line message
	LCD_Init();
	strcpy(LCD_Str, "DS8007 dual smart-");		// Create first line of LCD text
	temp = LCD_Curpos(1,1);						// Position cursor @ 1 line 1
	if (temp == 0) LCD_WRStr(LCD_Str);			// Write string to LCD
	else printf("\n!! LCD cursor address out of range !!\n");
	strcpy(LCD_Str, "card interface.");			// Create 2nd line of LCD text
	temp = LCD_Curpos(2,1);						// Position cursor @ 1 line 2
	if (temp == 0) LCD_WRStr(LCD_Str);			// Write string to LCD
	else printf("\n!! LCD cursor address out of range !!\n");
	temp = LCD_Curpos(2,20);					// Return cussor @ 20 line 2

	delay(2);  									// Delay for user to read message

	// Start main program loop
	while (1)
	{
		// Output initial instructions on LCD
		strcpy(LCD_Str, "Insert card into");
		LCD_WRCmd(DispClear);
		temp = LCD_Curpos (1,1);
		if (temp == 0) LCD_WRStr(LCD_Str);
		else printf("\n!! LCD cursor address out of range !!\n");
		strcpy(LCD_Str, "large socket below.");
		temp = LCD_Curpos (2,1);
		if (temp == 0) LCD_WRStr(LCD_Str);
		else printf("\n!! LCD cursor address out of range !!\n");

		printf("Waiting for smartcard presence\n");

		// Wait for card to be inserted
  		while (1)
  		{
    		val = dssc_readregister(MSR);
			if(temp != val){
	  			temp = val;	}
			if (val & MSR_PR1_MASK) {
				slot = 1;
      			break; }
    		if (val & MSR_PR2_MASK) {
	  			slot = 2;
	  			printf("Status check - 4\n");
      			break; }
		} // end while(1) card insert loop
  
		printf("Found smartcard in slot %d\n",slot);

		// Start main transaction loop
  		doTrans(slot);

		// Output ending message on LCD
		temp = transEnd(); 

		printf("Waiting for smartcard removal from slot %d\n",slot);
  		
		// Wait for card to be removed
		do
  		{val = dssc_readregister(MSR); }
  		while(( (val&MSR_PR1_MASK) != 0) || ( (val&MSR_PR2_MASK) != 0) );

  		printf("Smartcard removed.\n");

	} // end while (1) main program loop

	// The following code should never be executed
	// It is just an emergency catch.
  	printf("Exiting.\n");
  	while (1);
}

/* ************************************************************************** */
/* ************************************************************************** */
//
// Perform the ACOS3 transactions
//
void doTrans(int slot)
{

	int i, tmp ;

	// Set up for transaction and get an Answer To Reset
	dssc_selectcard(slot);
	i = dssc_powerup(POWERUP_ISO,POWERUP_5V);
	printf("Power up return: %d\n",i);
	if (i < 0) {
		printf("An error occurred at card power-up.\n");
		transError(); }
	
	// Get card ATR
	i = transATR() ;
	if (i < 0) {
		printf("An error occurred during the ATR.\n");
		transError(); }
	else printf("The card returned a good Answer To Reset.\n\n");

	// Display the current transaction Count and Balance
	i = selectFile();					// Select the Personalization file
	i = readRecords();					// Read the records containing Count and Balance
	dispTrans();						// Display the results

	// Create main transaction processing loop
	while (dssc_checkpresence(slot))	// Use presence detect as loop control
	{
		if (i >= 0)
		{
			// Cycle through tranction type options on LCD
			strcpy(LCD_Str, "For a Debit        ");
			LCD_WRCmd(DispClear);
			tmp = LCD_Curpos (1,1);
			if (tmp == 0) LCD_WRStr(LCD_Str);
			else printf("\n!! LCD cursor address out of range !!\n");
			strcpy(LCD_Str, "press SW4 now.     ");
			tmp = LCD_Curpos (2,1);
			if (tmp == 0) LCD_WRStr(LCD_Str);
			else printf("\n!! LCD cursor address out of range !!\n");
			tmp = selDelay(2);
			if (tmp == 1) {							// Perform a Debit transaction
				i = transDebit();
				printf("\nThe card was debited $10.00\n");
				break ; }

			if (!(dssc_checkpresence(slot))) break;	// Make sure the card is still inserted
			strcpy(LCD_Str, "For a Credit       ");
			tmp = LCD_Curpos (1,1);
			if (tmp == 0) LCD_WRStr(LCD_Str);
			else printf("\n!! LCD cursor address out of range !!\n");
			strcpy(LCD_Str, "press SW4 now.     ");
			tmp = LCD_Curpos (2,1);
			if (tmp == 0) LCD_WRStr(LCD_Str);
			else printf("\n!! LCD cursor address out of range !!\n");
			tmp = selDelay(2);
			if (tmp == 1) {							// Perform a Credit transaction
				i = transCredit();
				printf("\nThe card was credited $10.00\n");
				break ; }
		
			if (!(dssc_checkpresence(slot))) break;	// Make sure the card is still inserted
			strcpy(LCD_Str, "For Initialization ");
			tmp = LCD_Curpos (1,1);
			if (tmp == 0) LCD_WRStr(LCD_Str);
			else printf("\n!! LCD cursor address out of range !!\n");
			strcpy(LCD_Str, "press SW4 now.     ");
			tmp = LCD_Curpos (2,1);
			if (tmp == 0) LCD_WRStr(LCD_Str);
			else printf("\n!! LCD cursor address out of range !!\n");
			tmp = selDelay(2);
			if (tmp == 1) {							// Initialize the card
				i = initCard();
				printf("\nThe card was initialized.\n");
				break ; }
		} else {
			transError();
			break ;
		} // end IF i > 1
  	
	} // end while transloop 

	printf("\nThe transaction ended.\n");
	dssc_powerdown();

	return;

} // end doTrans()
//
/* ************************************************************************** */
//  Initialize the serial port for 38400 baud assuming a 14.7 MHz clock

void serialSetup()
{
	TMOD=0x21;		// Timer 1: 8-bit auto-reload from TH1, Timer 0: 16-bit
	TH1 = 0xFE;

#if (BAUDDOUBLE == 1)
  	PCON|=0x80;		// Turn on baud rate doubler SMOD_0
#else
	PCON&=0x7F;		// Turn off baud rate doubler SMOD_0
#endif

	TCON=0x50;		// Enable timers 0 and 1
	SCON=0x50;		// 10-bit async, enabled

  TI=1;
}

/* ************************************************************************** */
// Create a delay of "secs" seconds assuming a 14.7 MHz crystal
//
void delay(secs)
{
	
	int i, j, max ;

	max = secs * 10 ;
	for (i=0;i<max;i++) {
		for (j=0;j<24500;j++); }
}
/* ************************************************************************** */
// Create a delay of "secs" seconds assuming a 14.7 MHz crystal but testing
// for a press of SW4
//
uint8_t selDelay(secs)
{
	
	int k, l, m ;

	m = secs * 10;
	k = 0;
	while ((k < m) && (P32 == 1)) 
	{
		for (l=0;l<24500;l++);
		k++ ;
	}
	if (k >= m) return(0);
	else return(1);

} 